import java.util.ArrayList;

import org.apache.commons.math3.distribution.MultivariateNormalDistribution;

import cern.colt.matrix.DoubleMatrix1D;
import cern.colt.matrix.DoubleMatrix2D;
import cern.colt.matrix.DoubleFactory2D;
import cern.colt.matrix.doublealgo.Statistic;
import cern.colt.matrix.impl.DenseDoubleMatrix1D;
import cern.colt.matrix.impl.DenseDoubleMatrix2D;
import cern.colt.matrix.linalg.EigenvalueDecomposition;
import cern.colt.matrix.linalg.Algebra;


public class main {

	public static void main(String[] args) {
		int halfn = 3;
		int numreps = 2;
		int baselength = 2;
		
		Algebra a = new Algebra();
		
		double[] mu1 = new double[2*numreps];
		double[] mu2 = new double[2*numreps];
		
		for(int i = 0; i < mu1.length; ++i) mu1[i] = (i%2==0) ? 5 : 7;
		for(int i = 0; i < mu2.length; ++i) mu2[i] = (i%2==0) ? 7 : 5;
		
		DoubleMatrix1D mu1Mat = new DenseDoubleMatrix1D(mu1);
		DoubleMatrix1D mu2Mat = new DenseDoubleMatrix1D(mu2);
				
		double[] covDiag = new double[baselength*numreps];
		for(int i = 0; i < covDiag.length; ++i) covDiag[i] = .1;
		DoubleMatrix2D cov = DoubleFactory2D.dense.diagonal(new DenseDoubleMatrix1D(covDiag));
		
		MultivariateNormalDistribution mvn1 = new MultivariateNormalDistribution(mu1Mat.toArray(), cov.toArray());
		MultivariateNormalDistribution mvn2 = new MultivariateNormalDistribution(mu2Mat.toArray(), cov.toArray());
		
		double[][] odds = new double[halfn][mu1Mat.size()];
		double[][] evens = new double[halfn][mu2Mat.size()];
		
		for(int i = 0; i < halfn; ++i) {
			double[] sample = mvn1.sample();
			for(int j = 0; j < sample.length; ++j ) {
				odds[i][j] = sample[j];
			}
		}
		
		for(int i = 0; i < halfn; ++i) {
			double[] sample = mvn2.sample();
			for(int j = 0; j < sample.length; ++j ) {
				evens[i][j] = sample[j];
			}
		}
		
		DoubleMatrix2D oddsMat = new DenseDoubleMatrix2D(odds);
		DoubleMatrix2D evensMat = new DenseDoubleMatrix2D(evens);
		
		DoubleMatrix2D data = DoubleFactory2D.dense.appendRows(oddsMat, evensMat);
		
		double[][] meanCenterArr = data.toArray();
		//There really doesn't seem to be a better way to do this (?)
		//The source code for covariance does something similar...
		double[] colSums = new double[2*numreps];
		for (int i = 0; i < colSums.length; ++i) colSums[i] = 0.;
		for(int i = 0; i < meanCenterArr.length; i++) {
			for(int j = 0; j < meanCenterArr[i].length; ++j) {
				colSums[j] += meanCenterArr[i][j];
			}
		}
		for (int i = 0; i < colSums.length; ++i) colSums[i] /= (halfn*2);
		//Colsums are now column means
		//now to subtract...
		for(int i = 0; i < meanCenterArr.length; i++) {
			for(int j = 0; j < meanCenterArr[i].length; ++j) {
				meanCenterArr[i][j] -= colSums[j];
			}
		}
		
		DoubleMatrix2D meanCentered = new DenseDoubleMatrix2D(meanCenterArr);
		
		
		System.out.println("Raw data");
		System.out.println(data);
		
		System.out.println("Mean Centered Data");
		System.out.println(meanCentered);
		
		//Sanity check.
		//System.out.println(meanCentered.viewColumn(0).zSum());
		//Looks okay!
		
		DoubleMatrix2D covmat =  Statistic.covariance(data);
		System.out.println("Covariance Matrix");
		System.out.println(covmat);
		
		EigenvalueDecomposition ed = new EigenvalueDecomposition(covmat);
		DoubleMatrix1D eigVals = ed.getRealEigenvalues();
		//Rows are eigenvecs
		DoubleMatrix2D eigVecs = a.transpose(ed.getV());
		System.out.println("Eigenvalues");
		System.out.println(eigVals);
		System.out.println("Eigenvectors (in rows, sorted from least to greatest)");
		System.out.println(eigVecs);
		
		DoubleMatrix1D bigVec = eigVecs.viewRow(3);
		System.out.println("Norm of the evec associated w big eigv");
		System.out.println(a.norm2(bigVec));
		System.out.println("Checking that this pair of evecs are orthogonal");
		System.out.println(a.mult(eigVecs.viewRow(3),
					eigVecs.viewRow(2)));
		DoubleMatrix2D proj = new DenseDoubleMatrix2D(eigVecs.viewSelection(new int[]{3,2},
																			new int[]{0,1,2,3}).toArray());
		DoubleMatrix2D result = new DenseDoubleMatrix2D(halfn*2,2);
		meanCentered.zMult(a.transpose(proj), result);
		System.out.println("2D projection:");
		System.out.println(result);
	}
}
